#include "Mesh.h"
#include "intersect.h"
#include "util.h"

unsigned ambenv::MeshTriIntersect = 0;
unsigned ambenv::MeshTriCacheHits = 0;
unsigned ambenv::MeshTriCacheMisses = 0;
unsigned ambenv::MeshTriCacheSize = 20;

using namespace ambenv;

Mesh::Mesh( const std::string & name ) : Object( name )
{
}

void Mesh::addTriangle( const MeshTriangle & tri ) 
{
  m_tri.push_back( tri );
}

void Mesh::addVertex( const Vec3 & v, const Vec3 & n, const Vec3 & tc, const Vec3 & vcolor )
{
  m_vert.push_back( MeshVertex( v, n, tc, vcolor ) );
}

bool Mesh::intersect( const Ray & ray, Real epsilon )
{

  {
    std::list<MeshTriangle>::iterator itr = hit_cache.begin();
    for( ; itr != hit_cache.end(); ++itr ) {
      if( intersectTriangle( ray, *itr, epsilon ) ) {
	MeshTriangle head = *(hit_cache.begin());
	*(hit_cache.begin()) = *itr;
	*itr = head;
	MeshTriCacheHits++;
	return( true );
      }
    }
  }

  MeshTriCacheMisses++;

  {
    // -- for each triangle
    std::vector<MeshTriangle>::const_iterator itr = m_tri.begin();
    
    for( ; itr != m_tri.end(); ++itr ) {
      if( intersectTriangle( ray, *itr, epsilon ) ) {
	hit_cache.push_front( *itr );
	if( hit_cache.size() > MeshTriCacheSize ) {
	  hit_cache.pop_back();
	}
	return( true );
      }
    }
  }

  return( false );
}

bool Mesh::intersectTriangle( const Ray & ray, const MeshTriangle & tri, Real epsilon ) const
{
  MeshTriIntersect++;
  if( (tri.vert( 0 ) >= 0) && 
      (tri.vert( 1 ) >= 0) &&
      (tri.vert( 2 ) >= 0) ) {
	
      float orig[3], dir[3], vert0[3], vert1[3], vert2[3], a = 0.0, b = 0.0, c = 0.0;
      copy_to_farray( m_vert[tri.vert( 0 )].v(), vert0 );
      copy_to_farray( m_vert[tri.vert( 1 )].v(), vert1 );
      copy_to_farray( m_vert[tri.vert( 2 )].v(), vert2 );
      copy_to_farray( ray.getOrigin(), orig );
      copy_to_farray( ray.getDirection(), dir );

      if( intersect_triangle( orig, dir,
			      vert0, vert1, vert2,
			      &a, &b, &c ) == 1 ) {
	
	a = 1.0 - b - c;

        Vec3 point = m_vert[tri.vert( 0 )].v() * a +
          m_vert[tri.vert( 1 )].v() * b +
          m_vert[tri.vert( 2 )].v() * c;


        Vec3 d = ray.getDirection();
        Vec3 o = ray.getOrigin();
        Vec3 tv = (point - o);
        Real n = 0.0;
        if( d.x() != 0.0 ) {
          tv.x( tv.x() / d.x() );
          n++;
        } else {
          tv.x( 0.0 );
        }
        if( d.y() != 0.0 ) {
          tv.y( tv.y() / d.y() );
          n++;
        } else {
          tv.y( 0.0 );
        }
        if( d.z() != 0.0 ) {
          n++;
          tv.z( tv.z() / d.z() );
        } else {
          tv.z( 0.0 );
        }

        Real tval = REAL_MAX;
        if( n != 0.0 ) {
          tval = ( tv.x() + tv.y() + tv.z() ) / n;
        }

        // -- check if this intersection point is in valid range
        if( (tval > 0.0) && (tval > epsilon) ) {
	  return( true );
	}
      }
    }

    return( false );
}
