#ifndef __MESH_H_
#define __MESH_H_

#include "MeshElements.h"
#include "Object.h"
#include "Vec3.h"

#include <vector>
#include <string>
#include <cassert>

namespace ambenv {

  extern unsigned MeshTriIntersect;
  extern unsigned MeshTriCacheHits;
  extern unsigned MeshTriCacheMisses;
  extern unsigned MeshTriCacheSize;

  class Mesh : public Object {
  public:
    Mesh() {};
    Mesh( const std::string & name );

    bool intersect( const Ray & ray, Real epsilon = 0.0 );

    void addTriangle( const MeshTriangle & tri );
    void addVertex( const Vec3 & v, const Vec3 & n, 
		    const Vec3 & tc, 
		    const Vec3 & vcolor = Vec3() );

     unsigned numVertices() const 
     { 
       return( m_vert.size() ); 
     }
     unsigned numTriangles() const 
     { 
       return( m_tri.size() ); 
     }

     const MeshVertex & getVertex( unsigned i ) const {
       assert( i < m_vert.size() );
       return( m_vert[i] );
     }

     void setVertex( unsigned i, const MeshVertex & v )
     {
       assert( i < m_vert.size() );
       m_vert[i] = v;
     }

     const MeshTriangle & getTriangle( unsigned i ) const 
     {
       assert( i < m_tri.size() );
       return( m_tri[i] );
     }

     void clearHitCache()
     {
       hit_cache.clear();
     }

   private:

     bool intersectTriangle( const Ray & ray, 
			     const MeshTriangle & tri, 
			     Real epsilon = 0.0 ) const;

     std::vector<MeshVertex> m_vert;
     std::vector<MeshTriangle> m_tri;
     std::vector<MeshEdge> m_edge;

    std::list<MeshTriangle> hit_cache;

  };
  
}; /* namespace ambenv */

#endif /* __MESH_H_ */

