#ifndef __VEC3_H_
#define __VEC3_H_

#ifdef WIN32
#pragma warning(disable : 4305) /* Disable (truncation from 'const double' to 'float') warning */
#pragma warning(disable : 4244) /* Disable (conversion from 'double' to 'float', possible loss of data) warning */
#pragma warning(disable : 4786) /* Disable (dentifier was truncated to '255' characters in the debug information) warning */
#endif

#include <float.h>

namespace ambenv {

  typedef float Real;
  const Real REAL_MAX = FLT_MAX;
  const Real EPSILON = 1e-6;
  const Real REAL_PI = 3.14159265358979323846;

  class Vec3 {
  public:
    Vec3();
    Vec3( Real x, Real y, Real z );
    Vec3( const float * xyz );
    Vec3( const double* xyz );
    
    Real x() const;
    Real y() const;
    Real z() const;

    void x( Real x );
    void y( Real y );
    void z( Real z );

    //! vector operators
    Vec3 cross( const Vec3 & rhs ) const;
    Real dot( const Vec3 & rhs ) const;
    
    void normalize();
    Real length() const;
    Real lengthSq() const;
    Real sum() const;
    Real average( const Vec3 & weights ) const; //! get the weighted average

    bool approximates( const Vec3 & rhs ) const;
    
    bool operator==( const Vec3 & rhs ) const;
    bool operator!=( const Vec3 & rhs ) const;

    //! point-wise operators
    Vec3 power( Real exponent ) const;

    Vec3 operator+( const Vec3 & rhs ) const;
    Vec3 operator+( Real rhs ) const;
    void operator+=( const Vec3 & rhs );
    void operator+=( Real rhs );
  
    Vec3 operator-( const Vec3 & rhs ) const;
    Vec3 operator-( Real rhs ) const;
    void operator-=( const Vec3 & rhs );
    void operator-=( Real rhs );
    
    Vec3 operator*( const Vec3 & rhs ) const;
    Vec3 operator*( Real rhs ) const;
    void operator*=( const Vec3 & rhs );
    void operator*=( Real rhs );
  
    Vec3 operator/( const Vec3 & rhs ) const;
    Vec3 operator/( Real rhs ) const;
    void operator/=( const Vec3 & rhs );
    void operator/=( Real rhs );

  private:
    Real m_x;
    Real m_y;
    Real m_z;
  };

}; /* namespace ambenv */

#endif /* __VEC3_H_ */

