#ifndef __AMBENV_SDL_H_
#define __AMBENV_SDL_H_


#ifndef WIN32
#define GL_GLEXT_PROTOTYPES
#endif

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>


// -- We might have to get the function pointers for these extensions.
#ifdef WIN32
typedef void (APIENTRY * PFNGLACTIVETEXTUREPROC) (GLenum texture);
typedef void (APIENTRY * PFNGLMULTITEXCOORD2FPROC) (GLenum target, GLfloat s, GLfloat t);
PFNGLACTIVETEXTUREPROC glActiveTexture = NULL;
PFNGLMULTITEXCOORD2FPROC glMultiTexCoord2f = NULL;
#else
#ifndef GL_VERSION_1_3
#define glActiveTexture (glActiveTextureARB)
#define glMultiTexCoord2f (glMultiTexCoord2fARB)
#endif
#endif

// -- GLenum constants, core as of OpenGL 1.3.
// -- Cube mapping and multi-texturing.
#ifndef GL_VERSION_1_3
#define GL_TEXTURE_CUBE_MAP (GL_TEXTURE_CUBE_MAP_ARB)
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X (GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB)
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X (GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB)
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y (GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB)
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y (GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB)
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z (GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB)
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z (GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB)
#define GL_NORMAL_MAP (GL_NORMAL_MAP_ARB)
#define GL_REFLECTION_MAP (GL_REFLECTION_MAP_ARB)
#define GL_TEXTURE0 (GL_TEXTURE0_ARB)
#define GL_TEXTURE1 (GL_TEXTURE1_ARB)
#define GL_TEXTURE2 (GL_TEXTURE2_ARB)
#define GL_TEXTURE3 (GL_TEXTURE3_ARB)
#endif


const unsigned AE_DRAW_FLAT = 1;
const unsigned AE_DRAW_TEXTURE = 1 << 1;
const unsigned AE_DRAW_CUBE_MAP = 1 << 2;
const unsigned AE_DRAW_PVO = 1 << 3;
const unsigned AE_DRAW_BENT_NORMAL = 1 << 4;
const unsigned AE_DRAW_NORMALS = 1 << 7;
const unsigned AE_DRAW_EDGES = 1 << 8;


#ifdef __cplusplus
#include <string>
const unsigned NUM_CUBE_MAPS = 5;
const std::string g_cube_filename[NUM_CUBE_MAPS][6] = {
  {
    "data/cube_maps/amb128_left.tif",
    "data/cube_maps/amb128_right.tif",
    "data/cube_maps/amb128_top.tif",
    "data/cube_maps/amb128_bottom.tif",
    "data/cube_maps/amb128_front.tif",
    "data/cube_maps/amb128_back.tif"
  },
  {
    "data/cube_maps/amb256_left.tif",
    "data/cube_maps/amb256_right.tif",
    "data/cube_maps/amb256_top.tif",
    "data/cube_maps/amb256_bottom.tif",
    "data/cube_maps/amb256_front.tif",
    "data/cube_maps/amb256_back.tif"
  },
  {
    "data/cube_maps/amb512_left.tif",
    "data/cube_maps/amb512_right.tif",
    "data/cube_maps/amb512_top.tif",
    "data/cube_maps/amb512_bottom.tif",
    "data/cube_maps/amb512_front.tif",
    "data/cube_maps/amb512_back.tif"
  },
  {  
    "data/cube_maps/grove128_left.tif",
    "data/cube_maps/grove128_right.tif",
    "data/cube_maps/grove128_top.tif",
    "data/cube_maps/grove128_bottom.tif",
    "data/cube_maps/grove128_front.tif",
    "data/cube_maps/grove128_back.tif"
  },
  {  
    "data/cube_maps/rb_left.tif",
    "data/cube_maps/rb_right.tif",
    "data/cube_maps/rb_top.tif",
    "data/cube_maps/rb_bottom.tif",
    "data/cube_maps/rb_front.tif",
    "data/cube_maps/rb_back.tif"
  }
};

const GLenum g_cube_face[6] = {
  GL_TEXTURE_CUBE_MAP_POSITIVE_X,
  GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
  GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
  GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
  GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
  GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
};

const unsigned g_num_ext_names = 12;
const std::string g_ext_names[g_num_ext_names] = {
  "GL_ARB_multitexture",
  "GL_EXT_bgra",
  "GL_ARB_texture_cube_map",
  "GL_EXT_texture_cube_map",
  "GL_EXT_compiled_vertex_array",
  "GL_SGIX_depth_texture",
  "GL_SGIX_shadow",
  "GL_NV_register_combiners",
  "GL_EXT_texture_env_combine",
  "GL_EXT_texture_edge_clamp",
  "GL_EXT_separate_specular_color",
  "GL_NV_texture_rectangle"
};

#endif

#endif /* __AMBENV_SDL_H_ */
