#include "Vec3.h"
#include "Matrix4x4.h"
#include "Mesh.h"
#include "Ray.h"
#include "Scene.h"

#include <iostream>
#include <string>
#include <vector>

namespace ambenv {

  std::ostream & operator<<( std::ostream & output, const Vec3 & vec );
  std::ostream & operator<<( std::ostream & output, const Matrix4x4 & mat );
  std::ostream & operator<<( std::ostream & output, const Ray & ray );
  std::ostream & operator<<( std::ostream & output, const Scene & scene );
  std::ostream & operator<<( std::ostream & output, const Object * object );
  std::ostream & operator<<( std::ostream & output, const Mesh * mesh );

  bool write_rgb_tiff( const std::string & filename, int w, int h, unsigned char * data );
  bool read_rgba_tiff( const std::string & filename, unsigned & w, unsigned & h, 
		       std::vector<unsigned char> & buffer );

  unsigned power_of_two( unsigned input );
  unsigned char convert_to_uchar( Real v );

  void copy_to_farray( const Vec3 & v, float a[3] );

  std::string get_path( const std::string filename );

  //! Compute the area of a 3d-triangle
  /*! The normal must be a unit normal or this will not work.
   *  See "Geometric Tools for Computer Graphics:
   *  Area of 3D Polygon by Projection" for reference.
   */
  Real get_tri_area( const Vec3 vertex[3], const Vec3 & normal );

  void generate_vertex_normals( Mesh & mesh );

  void doGLMultMatrix( const Matrix4x4 & mat );
  
}; /* namespace ambenv */ 
